@@ -6,25 +6,28 @@ from pythainlp.tokenize import (
     DEFAULT_WORD_DICT_TRIE,
     Tokenizer,
     attacut,
+    crfcls,
     clause_tokenize,
-)
-from pythainlp.tokenize import deepcut as tokenize_deepcut
-from pythainlp.tokenize import etcc, longest, multi_cut, newmm
-from pythainlp.tokenize import pyicu as tokenize_pyicu
-from pythainlp.tokenize import (
+    deepcut,
+    etcc,
+    longest,
+    multi_cut,
+    nercut,
+    newmm,
+    pyicu,
     sent_tokenize,
+    ssg,
     subword_tokenize,
     syllable_tokenize,
     tcc,
     word_tokenize,
 )
-from pythainlp.tokenize.ssg import segment as ssg_segment
 from pythainlp.util import dict_trie
 
 
 class TestTokenizePackage(unittest.TestCase):
     def setUp(self):
-        self.text_1 = "หมอนทองตากลมหูว์MBK39"
+        self.text_1 = "หมอนทองตากลมหูว์MBK39 :.ฉฺ๐๐๓-#™±"
         self.text_2 = "ทดสอบ"
 
         self.long_text = (
@@ -189,180 +192,22 @@ class TestTokenizePackage(unittest.TestCase):
             "กกกกกกกกกกกกกกกกกกกกกกกกกกกกกกกกกกกกกก"
         )
 
+    def test_Tokenizer(self):
+        _tokenizer = Tokenizer(DEFAULT_WORD_DICT_TRIE)
+        self.assertEqual(_tokenizer.word_tokenize(""), [])
+        _tokenizer.set_tokenize_engine("longest")
+        self.assertEqual(_tokenizer.word_tokenize(None), [])
+
+        _tokenizer = Tokenizer()
+        self.assertEqual(_tokenizer.word_tokenize("ก"), ["ก"])
+
     def test_clause_tokenize(self):
+        self.assertEqual(clause_tokenize(None), [])
+        self.assertEqual(clause_tokenize(""), [])
         self.assertIsNotNone(clause_tokenize(["ฉัน", "ทดสอบ"]))
         self.assertIsInstance(clause_tokenize(["ฉัน", "ทดสอบ"]), list)
-
-    def test_Tokenizer(self):
-        t_test = Tokenizer(DEFAULT_WORD_DICT_TRIE)
-        self.assertEqual(t_test.word_tokenize(""), [])
-        t_test.set_tokenize_engine("longest")
-        self.assertEqual(t_test.word_tokenize(None), [])
-
-        t_test = Tokenizer()
-        self.assertEqual(t_test.word_tokenize("ก"), ["ก"])
-
-    def test_etcc(self):
-        self.assertEqual(etcc.segment(None), [])
-        self.assertEqual(etcc.segment(""), [])
-        self.assertIsInstance(etcc.segment("คืนความสุข"), list)
-        self.assertEqual(
-            etcc.segment("หาเงินเพื่อเรียน"),
-            ["หา", "เงิน", "เพื่", "อ", "เรีย", "น"],
-        )
-        self.assertEqual(etcc.segment("หนังสือ"), ["ห", "นัง", "สือ"])
-        self.assertIsNotNone(
-            etcc.segment(
-                "หมูแมวเหล่านี้ด้วยเหตุผลเชื่อมโยงทางกรรมพันธุ์"
-                + "สัตว์มีแขนขาหน้าหัวเราะเพราะแข็งขืน"
-            )
-        )
-
-    def test_word_tokenize(self):
-        self.assertEqual(word_tokenize(""), [])
-        self.assertEqual(
-            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย"),
-            ["ฉัน", "รัก", "ภาษาไทย", "เพราะ", "ฉัน", "เป็น", "คนไทย"],
-        )
-        self.assertIsNotNone(word_tokenize(self.text_1, engine="newmm"))
-        self.assertIsNotNone(word_tokenize(self.text_1, engine="mm"))
-        self.assertIsNotNone(word_tokenize(self.text_1, engine="longest"))
-        self.assertIsNotNone(word_tokenize(self.text_1, engine="icu"))
-        self.assertIsNotNone(word_tokenize(self.text_1, engine="deepcut"))
-        self.assertIsNotNone(word_tokenize(self.text_1, engine="attacut"))
-        with self.assertRaises(ValueError):
-            word_tokenize("หมอนทอง", engine="XX")  # engine does not exist
-
-        self.assertTrue(
-            "ไฟ" in word_tokenize("รถไฟฟ้า", custom_dict=dict_trie(["ไฟ"]))
-        )
-
-    def test_word_tokenize_deepcut(self):
-        self.assertEqual(tokenize_deepcut.segment(None), [])
-        self.assertEqual(tokenize_deepcut.segment(""), [])
-        self.assertIsNotNone(
-            tokenize_deepcut.segment("ทดสอบ", DEFAULT_WORD_DICT_TRIE)
-        )
-        self.assertIsNotNone(tokenize_deepcut.segment("ทดสอบ", ["ทด", "สอบ"]))
-        self.assertIsNotNone(word_tokenize("ทดสอบ", engine="deepcut"))
-        self.assertIsNotNone(
-            word_tokenize(
-                "ทดสอบ", engine="deepcut", custom_dict=DEFAULT_WORD_DICT_TRIE
-            )
-        )
-
-    def test_word_tokenize_icu(self):
-        self.assertEqual(tokenize_pyicu.segment(None), [])
-        self.assertEqual(tokenize_pyicu.segment(""), [])
-        self.assertEqual(
-            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="icu"),
-            ["ฉัน", "รัก", "ภาษา", "ไทย", "เพราะ", "ฉัน", "เป็น", "คน", "ไทย"],
-        )
-
-    def test_word_tokenize_longest(self):
-        self.assertEqual(longest.segment(None), [])
-        self.assertEqual(longest.segment(""), [])
-        self.assertIsInstance(
-            longest.segment("กรุงเทพฯมากๆเพราโพาง BKKฯ"), list
-        )
-        self.assertEqual(
-            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="longest"),
-            ["ฉัน", "รัก", "ภาษาไทย", "เพราะ", "ฉัน", "เป็น", "คนไทย"],
-        )
-        longest_tokenizer = Tokenizer(["ปวด", "เฉียบ", "พลัน", "เฉียบพลัน"])
-        self.assertEqual(
-            longest_tokenizer.word_tokenize("ปวดเฉียบพลัน"),
-            ["ปวด", "เฉียบพลัน"],
-        )
-        self.assertEqual(
-            longest_tokenizer.word_tokenize("เฉียบพลัน"), ["เฉียบพลัน"],
-        )
-
-    def test_word_tokenize_mm(self):
-        self.assertEqual(multi_cut.segment(None), [])
-        self.assertEqual(multi_cut.segment(""), [])
-        self.assertIsNotNone(multi_cut.segment("ตัด", dict_trie([""])))
-
-        self.assertEqual(word_tokenize("", engine="mm"), [])
-        self.assertEqual(
-            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="mm"),
-            ["ฉัน", "รัก", "ภาษาไทย", "เพราะ", "ฉัน", "เป็น", "คนไทย"],
-        )
-
-        self.assertIsNotNone(multi_cut.mmcut("ทดสอบ"))
-
-        self.assertIsNotNone(
-            multi_cut.find_all_segment("รถไฟฟ้ากรุงเทพมหานครBTS")
-        )
-        self.assertEqual(multi_cut.find_all_segment(None), [])
-
-    def test_word_tokenize_newmm(self):
-        self.assertEqual(newmm.segment(None), [])
-        self.assertEqual(newmm.segment(""), [])
-        self.assertEqual(
-            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="newmm"),
-            ["ฉัน", "รัก", "ภาษาไทย", "เพราะ", "ฉัน", "เป็น", "คนไทย"],
-        )
-        self.assertEqual(
-            word_tokenize(
-                "สวัสดีครับ สบายดีไหมครับ",
-                engine="newmm",
-                keep_whitespace=True,
-            ),
-            ["สวัสดี", "ครับ", " ", "สบายดี", "ไหม", "ครับ"],
-        )
-        self.assertEqual(
-            word_tokenize("จุ๋มง่วงนอนยัง", engine="newmm"),
-            ["จุ๋ม", "ง่วงนอน", "ยัง"],
-        )
-        self.assertEqual(
-            word_tokenize("จุ๋มง่วง", engine="newmm"), ["จุ๋ม", "ง่วง"]
-        )
-        self.assertEqual(
-            word_tokenize(
-                "จุ๋ม   ง่วง", engine="newmm", keep_whitespace=False
-            ),
-            ["จุ๋ม", "ง่วง"],
-        )
-        self.assertFalse(
-            " " in word_tokenize("จุ๋มง่วง", keep_whitespace=False,)
-        )
-
-    def test_word_tokenize_newmm_longtext(self):
-        self.assertIsInstance(
-            word_tokenize(self.long_text, engine="newmm"), list
-        )
-        self.assertIsInstance(
-            word_tokenize(self.long_text, engine="newmm-safe"), list
-        )
-
-    def test_word_tokenize_newmm_dangertext(self):
-        self.assertIsInstance(
-            word_tokenize(self.danger_text1, engine="newmm"), list
-        )
-        self.assertIsInstance(
-            word_tokenize(self.danger_text2, engine="newmm"), list
-        )
-        self.assertIsInstance(
-            word_tokenize(self.danger_text3, engine="newmm"), list
-        )
-        self.assertIsInstance(
-            word_tokenize(self.danger_text1, engine="newmm-safe"), list
-        )
-        self.assertIsInstance(
-            word_tokenize(self.danger_text2, engine="newmm-safe"), list
-        )
-        self.assertIsInstance(
-            word_tokenize(self.danger_text3, engine="newmm-safe"), list
-        )
-
-    def test_word_tokenize_attacut(self):
-        self.assertEqual(attacut.segment(None), [])
-        self.assertEqual(attacut.segment(""), [])
-        self.assertEqual(
-            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="attacut"),
-            ["ฉัน", "รัก", "ภาษา", "ไทย", "เพราะ", "ฉัน", "เป็น", "คน", "ไทย"],
-        )
+        self.assertIsNotNone(crfcls.segment(["ฉัน", "ทดสอบ"]))
+        self.assertIsInstance(crfcls.segment(["ฉัน", "ทดสอบ"]), list)
 
     def test_sent_tokenize(self):
         self.assertEqual(sent_tokenize(None), [])
@@ -423,13 +268,6 @@ class TestTokenizePackage(unittest.TestCase):
         with self.assertRaises(ValueError):
             sent_tokenize("ฉันไป กิน", engine="XX")  # engine does not exist
 
-    def test_ssg_tokenize(self):
-        self.assertEqual(ssg_segment(None), [])
-        self.assertEqual(ssg_segment(""), [])
-        self.assertTrue(
-            "ดาว" in syllable_tokenize("สวัสดีดาวอังคาร", engine="ssg")
-        )
-
     def test_subword_tokenize(self):
         self.assertEqual(subword_tokenize(None), [])
         self.assertEqual(subword_tokenize(""), [])
@@ -448,6 +286,17 @@ class TestTokenizePackage(unittest.TestCase):
             "า" in subword_tokenize("สวัสดีดาวอังคาร", engine="etcc")
         )
         self.assertIsInstance(subword_tokenize("โควิด19", engine="etcc"), list)
+        self.assertEqual(subword_tokenize(None, engine="wangchanberta"), [])
+        self.assertEqual(subword_tokenize("", engine="wangchanberta"), [])
+        self.assertIsInstance(
+            subword_tokenize("สวัสดิีดาวอังคาร", engine="wangchanberta"), list
+        )
+        self.assertFalse(
+            "า" in subword_tokenize("สวัสดีดาวอังคาร", engine="wangchanberta")
+        )
+        self.assertIsInstance(
+            subword_tokenize("โควิด19", engine="wangchanberta"), list
+        )
         self.assertFalse(
             " " in subword_tokenize("พันธมิตร ชา นม", keep_whitespace=False)
         )
@@ -478,6 +327,193 @@ class TestTokenizePackage(unittest.TestCase):
         with self.assertRaises(ValueError):
             syllable_tokenize("กรอเทป", engine="XX")  # engine does not exist
 
+    def test_word_tokenize(self):
+        self.assertEqual(word_tokenize(""), [])
+        self.assertEqual(
+            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย"),
+            ["ฉัน", "รัก", "ภาษาไทย", "เพราะ", "ฉัน", "เป็น", "คนไทย"],
+        )
+        self.assertIsNotNone(word_tokenize(self.text_1, engine="attacut"))
+        self.assertIsNotNone(word_tokenize(self.text_1, engine="deepcut"))
+        self.assertIsNotNone(word_tokenize(self.text_1, engine="icu"))
+        self.assertIsNotNone(word_tokenize(self.text_1, engine="longest"))
+        self.assertIsNotNone(word_tokenize(self.text_1, engine="mm"))
+        self.assertIsNotNone(word_tokenize(self.text_1, engine="nercut"))
+        self.assertIsNotNone(word_tokenize(self.text_1, engine="newmm"))
+
+        with self.assertRaises(ValueError):
+            word_tokenize("หมอนทอง", engine="XX")  # engine does not exist
+
+        self.assertTrue(
+            "ไฟ" in word_tokenize("รถไฟฟ้า", custom_dict=dict_trie(["ไฟ"]))
+        )
+
+    def test_attacut(self):
+        self.assertEqual(attacut.segment(None), [])
+        self.assertEqual(attacut.segment(""), [])
+        self.assertEqual(
+            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="attacut"),
+            ["ฉัน", "รัก", "ภาษา", "ไทย", "เพราะ", "ฉัน", "เป็น", "คน", "ไทย"],
+        )
+        self.assertEqual(
+            attacut.segment(
+                "ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", model="attacut-sc"
+            ),
+            ["ฉัน", "รัก", "ภาษา", "ไทย", "เพราะ", "ฉัน", "เป็น", "คน", "ไทย"],
+        )
+        self.assertIsNotNone(
+            attacut.segment(
+                "ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", model="attacut-c"
+            )
+        )
+
+    def test_deepcut(self):
+        self.assertEqual(deepcut.segment(None), [])
+        self.assertEqual(deepcut.segment(""), [])
+        self.assertIsNotNone(deepcut.segment("ทดสอบ", DEFAULT_WORD_DICT_TRIE))
+        self.assertIsNotNone(deepcut.segment("ทดสอบ", ["ทด", "สอบ"]))
+        self.assertIsNotNone(word_tokenize("ทดสอบ", engine="deepcut"))
+        self.assertIsNotNone(
+            word_tokenize(
+                "ทดสอบ", engine="deepcut", custom_dict=DEFAULT_WORD_DICT_TRIE
+            )
+        )
+
+    def test_etcc(self):
+        self.assertEqual(etcc.segment(None), [])
+        self.assertEqual(etcc.segment(""), [])
+        self.assertIsInstance(etcc.segment("คืนความสุข"), list)
+        self.assertEqual(
+            etcc.segment("หาเงินเพื่อเรียน"),
+            ["หา", "เงิน", "เพื่", "อ", "เรีย", "น"],
+        )
+        self.assertEqual(etcc.segment("หนังสือ"), ["ห", "นัง", "สือ"])
+        self.assertIsNotNone(
+            etcc.segment(
+                "หมูแมวเหล่านี้ด้วยเหตุผลเชื่อมโยงทางกรรมพันธุ์"
+                + "สัตว์มีแขนขาหน้าหัวเราะเพราะแข็งขืน"
+            )
+        )
+
+    def test_icu(self):
+        self.assertEqual(pyicu.segment(None), [])
+        self.assertEqual(pyicu.segment(""), [])
+        self.assertEqual(
+            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="icu"),
+            ["ฉัน", "รัก", "ภาษา", "ไทย", "เพราะ", "ฉัน", "เป็น", "คน", "ไทย"],
+        )
+
+    def test_longest(self):
+        self.assertEqual(longest.segment(None), [])
+        self.assertEqual(longest.segment(""), [])
+        self.assertIsInstance(
+            longest.segment("กรุงเทพฯมากๆเพราโพาง BKKฯ"), list
+        )
+        self.assertEqual(
+            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="longest"),
+            ["ฉัน", "รัก", "ภาษาไทย", "เพราะ", "ฉัน", "เป็น", "คนไทย"],
+        )
+        longest_tokenizer = Tokenizer(["ปวด", "เฉียบ", "พลัน", "เฉียบพลัน"])
+        self.assertEqual(
+            longest_tokenizer.word_tokenize("ปวดเฉียบพลัน"),
+            ["ปวด", "เฉียบพลัน"],
+        )
+        self.assertEqual(
+            longest_tokenizer.word_tokenize("เฉียบพลัน"), ["เฉียบพลัน"],
+        )
+
+    def test_mm(self):
+        self.assertEqual(multi_cut.segment(None), [])
+        self.assertEqual(multi_cut.segment(""), [])
+        self.assertIsNotNone(multi_cut.segment("ตัด", dict_trie([""])))
+
+        self.assertEqual(word_tokenize("", engine="mm"), [])
+        self.assertEqual(
+            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="mm"),
+            ["ฉัน", "รัก", "ภาษาไทย", "เพราะ", "ฉัน", "เป็น", "คนไทย"],
+        )
+
+        self.assertIsNotNone(multi_cut.mmcut("ทดสอบ"))
+
+        self.assertIsNotNone(
+            multi_cut.find_all_segment("รถไฟฟ้ากรุงเทพมหานครBTS")
+        )
+        self.assertEqual(multi_cut.find_all_segment(None), [])
+
+    def test_newmm(self):
+        self.assertEqual(newmm.segment(None), [])
+        self.assertEqual(newmm.segment(""), [])
+        self.assertEqual(
+            word_tokenize("ฉันรักภาษาไทยเพราะฉันเป็นคนไทย", engine="newmm"),
+            ["ฉัน", "รัก", "ภาษาไทย", "เพราะ", "ฉัน", "เป็น", "คนไทย"],
+        )
+        self.assertEqual(
+            word_tokenize(
+                "สวัสดีครับ สบายดีไหมครับ",
+                engine="newmm",
+                keep_whitespace=True,
+            ),
+            ["สวัสดี", "ครับ", " ", "สบายดี", "ไหม", "ครับ"],
+        )
+        self.assertEqual(
+            word_tokenize("จุ๋มง่วงนอนยัง", engine="newmm"),
+            ["จุ๋ม", "ง่วงนอน", "ยัง"],
+        )
+        self.assertEqual(
+            word_tokenize("จุ๋มง่วง", engine="newmm"), ["จุ๋ม", "ง่วง"]
+        )
+        self.assertEqual(
+            word_tokenize(
+                "จุ๋ม   ง่วง", engine="newmm", keep_whitespace=False
+            ),
+            ["จุ๋ม", "ง่วง"],
+        )
+        self.assertFalse(
+            " " in word_tokenize("จุ๋มง่วง", keep_whitespace=False,)
+        )
+
+    def test_newmm_longtext(self):
+        self.assertIsInstance(
+            word_tokenize(self.long_text, engine="newmm"), list
+        )
+        self.assertIsInstance(
+            word_tokenize(self.long_text, engine="newmm-safe"), list
+        )
+
+    def test_newmm_dangertext(self):
+        self.assertIsInstance(
+            word_tokenize(self.danger_text1, engine="newmm"), list
+        )
+        self.assertIsInstance(
+            word_tokenize(self.danger_text2, engine="newmm"), list
+        )
+        self.assertIsInstance(
+            word_tokenize(self.danger_text3, engine="newmm"), list
+        )
+        self.assertIsInstance(
+            word_tokenize(self.danger_text1, engine="newmm-safe"), list
+        )
+        self.assertIsInstance(
+            word_tokenize(self.danger_text2, engine="newmm-safe"), list
+        )
+        self.assertIsInstance(
+            word_tokenize(self.danger_text3, engine="newmm-safe"), list
+        )
+
+    def test_nercut(self):
+        self.assertEqual(nercut.segment(None), [])
+        self.assertEqual(nercut.segment(""), [])
+        self.assertIsNotNone(nercut.segment("ทดสอบ"))
+        self.assertIsNotNone(nercut.segment("ทดสอบ"))
+        self.assertIsNotNone(word_tokenize("ทดสอบ", engine="nercut"))
+
+    def test_ssg(self):
+        self.assertEqual(ssg.segment(None), [])
+        self.assertEqual(ssg.segment(""), [])
+        self.assertTrue(
+            "ดาว" in syllable_tokenize("สวัสดีดาวอังคาร", engine="ssg")
+        )
+
     def test_tcc(self):
         self.assertEqual(tcc.segment(None), [])
         self.assertEqual(tcc.segment(""), [])
